@@ -7,18 +7,21 @@
 
 CircuitPython library for the TI TMP117 Temperature sensor
 
-* Author(s): Bryan Siepert
+* Author(s): Bryan Siepert, Ian Grant
 
 parts based on SparkFun_TMP117_Arduino_Library by Madison Chodikov @ SparkFun Electronics:
 https://github.com/sparkfunX/Qwiic_TMP117
 https://github.com/sparkfun/SparkFun_TMP117_Arduino_Library
 
+Serial number register information:
+https://e2e.ti.com/support/sensors/f/1023/t/815716?TMP117-Reading-Serial-Number-from-EEPROM
+
 Implementation Notes
 --------------------
 
 **Hardware:**
 
-* `Adafruit TMP117 Breakout <https:#www.adafruit.com/product/PID_HERE>`_
+* `Adafruit TMP117 Breakout <https:#www.adafruit.com/product/4821>`_
 
 **Software and Dependencies:**
 
@@ -65,6 +68,18 @@ _SHUTDOWN_MODE = 0b01  # Shutdown Conversion Mode
 AlertStatus = namedtuple("AlertStatus", ["high_alert", "low_alert"])
 
 
+def _convert_to_integer(bytes_to_convert):
+    """Use bitwise operators to convert the bytes into integers."""
+    integer = None
+    for chunk in bytes_to_convert:
+        if not integer:
+            integer = chunk
+        else:
+            integer = integer << 8
+            integer = integer | chunk
+    return integer
+
+
 class CV:
     """struct helper"""
 
@@ -342,43 +357,45 @@ class TMP117:
 
     @property
     def measurement_mode(self):
+        # pylint: disable=line-too-long
         """Sets the measurement mode, specifying the behavior of how often measurements are taken.
-        `measurement_mode` must be one of:
+                `measurement_mode` must be one of:
 
-+----------------------------------------+------------------------------------------------------+
-| Mode                                   | Behavior                                             |
-+========================================+======================================================+
-| :py:const:`MeasurementMode.CONTINUOUS` | Measurements are made at the interval determined by  |
-|                                        |                                                      |
-|                                        | `averaged_measurements` and `measurement_delay`.     |
-|                                        |                                                      |
-|                                        | `temperature` returns the most recent measurement    |
-+----------------------------------------+------------------------------------------------------+
-| :py:const:`MeasurementMode.ONE_SHOT`   | Take a single measurement with the current number of |
-|                                        |                                                      |
-|                                        | `averaged_measurements` and switch to                |
-|                                        | :py:const:`SHUTDOWN` when                            |
-|                                        |                                                      |
-|                                        | finished.                                            |
-|                                        |                                                      |
-|                                        |                                                      |
-|                                        | `temperature` will return the new measurement until  |
-|                                        |                                                      |
-|                                        | `measurement_mode` is set to :py:const:`CONTINUOUS`  |
-|                                        | or :py:const:`ONE_SHOT` is                           |
-|                                        |                                                      |
-|                                        | set again.                                           |
-+----------------------------------------+------------------------------------------------------+
-| :py:const:`MeasurementMode.SHUTDOWN`   | The sensor is put into a low power state and no new  |
-|                                        |                                                      |
-|                                        | measurements are taken.                              |
-|                                        |                                                      |
-|                                        | `temperature` will return the last measurement until |
-|                                        |                                                      |
-|                                        | a new `measurement_mode` is selected.                |
-+----------------------------------------+------------------------------------------------------+
+        +----------------------------------------+------------------------------------------------------+
+        | Mode                                   | Behavior                                             |
+        +========================================+======================================================+
+        | :py:const:`MeasurementMode.CONTINUOUS` | Measurements are made at the interval determined by  |
+        |                                        |                                                      |
+        |                                        | `averaged_measurements` and `measurement_delay`.     |
+        |                                        |                                                      |
+        |                                        | `temperature` returns the most recent measurement    |
+        +----------------------------------------+------------------------------------------------------+
+        | :py:const:`MeasurementMode.ONE_SHOT`   | Take a single measurement with the current number of |
+        |                                        |                                                      |
+        |                                        | `averaged_measurements` and switch to                |
+        |                                        | :py:const:`SHUTDOWN` when                            |
+        |                                        |                                                      |
+        |                                        | finished.                                            |
+        |                                        |                                                      |
+        |                                        |                                                      |
+        |                                        | `temperature` will return the new measurement until  |
+        |                                        |                                                      |
+        |                                        | `measurement_mode` is set to :py:const:`CONTINUOUS`  |
+        |                                        | or :py:const:`ONE_SHOT` is                           |
+        |                                        |                                                      |
+        |                                        | set again.                                           |
+        +----------------------------------------+------------------------------------------------------+
+        | :py:const:`MeasurementMode.SHUTDOWN`   | The sensor is put into a low power state and no new  |
+        |                                        |                                                      |
+        |                                        | measurements are taken.                              |
+        |                                        |                                                      |
+        |                                        | `temperature` will return the last measurement until |
+        |                                        |                                                      |
+        |                                        | a new `measurement_mode` is selected.                |
+        +----------------------------------------+------------------------------------------------------+
 
         """
+        # pylint: enable=line-too-long
         return self._mode
 
     @measurement_mode.setter
@@ -470,6 +487,31 @@ class TMP117:
             raise AttributeError("alert_mode must be an `AlertMode`")
         self._raw_alert_mode = value
 
+    @property
+    def serial_number(self):
+        """A 48-bit, factory-set unique identifier for the device."""
+        eeprom1_data = bytearray(2)
+        eeprom2_data = bytearray(2)
+        eeprom3_data = bytearray(2)
+        # Fetch EEPROM registers
+        with self.i2c_device as i2c:
+            i2c.write_then_readinto(bytearray([_EEPROM1]), eeprom1_data)
+            i2c.write_then_readinto(bytearray([_EEPROM2]), eeprom2_data)
+            i2c.write_then_readinto(bytearray([_EEPROM3]), eeprom3_data)
+        # Combine the 2-byte portions
+        combined_id = bytearray(
+            [
+                eeprom1_data[0],
+                eeprom1_data[1],
+                eeprom2_data[0],
+                eeprom2_data[1],
+                eeprom3_data[0],
+                eeprom3_data[1],
+            ]
+        )
+        # Convert to an integer
+        return _convert_to_integer(combined_id)
+
     def _set_mode_and_wait_for_measurement(self, mode):
 
         self._mode = mode
